@@ -1,8 +1,6 @@
 /* Basic movement functions
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Sandro Sigala.
-   Copyright (c) 2003, 2004 Reuben Thomas.
+   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -29,7 +27,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "zile.h"
+#include "main.h"
 #include "extern.h"
 
 /* Goal-column to arrive when `prev/next-line' functions are used.  */
@@ -46,8 +44,6 @@ Move point to beginning of current line.
      `prev/next-line' calls.  */
   thisflag |= FLAG_DONE_CPCN;
   cur_goalc = 0;
-
-  return leT;
 }
 END_DEFUN
 
@@ -62,8 +58,6 @@ Move point to end of current line.
      `prev/next-line' calls.  */
   thisflag |= FLAG_DONE_CPCN;
   cur_goalc = INT_MAX;
-
-  return leT;
 }
 END_DEFUN
 
@@ -149,13 +143,13 @@ the cursor is positioned after the character in that line which spans this
 column, or at the end of the line if it is not long enough.
 +*/
 {
-  int i;
-
   if (uniarg < 0)
     return FUNCALL_ARG (next_line, -uniarg);
 
   if (!bobp ())
     {
+      int i;
+
       for (i = 0; i < uniarg; i++)
         if (!previous_line ())
           {
@@ -166,8 +160,6 @@ column, or at the end of the line if it is not long enough.
     }
   else if (lastflag & FLAG_DONE_CPCN)
     thisflag |= FLAG_DONE_CPCN;
-
-  return leT;
 }
 END_DEFUN
 
@@ -200,13 +192,13 @@ the cursor is positioned after the character in that line which spans this
 column, or at the end of the line if it is not long enough.
 +*/
 {
-  int i;
-
   if (uniarg < 0)
     return FUNCALL_ARG (previous_line, -uniarg);
 
   if (!eobp ())
     {
+      int i;
+
       for (i = 0; i < uniarg; i++)
         if (!next_line ())
           {
@@ -219,61 +211,76 @@ column, or at the end of the line if it is not long enough.
     }
   else if (lastflag & FLAG_DONE_CPCN)
     thisflag |= FLAG_DONE_CPCN;
-
-  return leT;
 }
 END_DEFUN
 
-DEFUN ("goto-char", goto_char)
+DEFUN_ARGS ("goto-char", goto_char,
+            INT_ARG (n))
 /*+
 Read a number N and move the cursor to character number N.
 Position 1 is the beginning of the buffer.
 +*/
 {
-  int to_char, count;
+  INT_INIT (n)
+  else
+    do
+      {
+        char *ms = minibuf_read ("Goto char: ", "");
+        if (ms == NULL)
+          {
+            ok = FUNCALL (keyboard_quit);
+            break;
+          }
+        n = strtoul (ms, NULL, 10);
+        if (n == LONG_MAX)
+          ding ();
+        free (ms);
+      }
+    while (n == LONG_MAX);
 
-  do
+  if (ok == leT && n != LONG_MAX)
     {
-      char *ms = minibuf_read ("Goto char: ", "");
-      if (ms == NULL)
-        return FUNCALL (keyboard_quit);
-      to_char = atoi (ms);
-      free (ms);
-      if (to_char < 0)
-        ding ();
+      long count;
+
+      gotobob ();
+      for (count = 1; count < n; count++)
+        if (!forward_char ())
+          break;
     }
-  while (to_char < 0);
 
-  gotobob ();
-  for (count = 1; count < to_char; ++count)
-    if (!forward_char ())
-      break;
-
-  return leT;
+  INT_FREE (n);
 }
 END_DEFUN
 
-DEFUN ("goto-line", goto_line)
+DEFUN_ARGS ("goto-line", goto_line,
+            INT_ARG (n))
 /*+
-Move cursor to the beginning of the specified line.
-Line 1 is the beginning of the buffer.
+Goto line arg, counting from line 1 at beginning of buffer.
 +*/
 {
-  size_t to_line = minibuf_read_number ("Goto line: ");
+  INT_INIT (n)
+  else
+    {
+      n = minibuf_read_number ("Goto line: ");
+      if (n == LONG_MAX - 1)
+        minibuf_error ("End of file during parsing");
+    }
 
-  if (to_line == SIZE_MAX)
-    return leNIL;
-  else if (to_line == SIZE_MAX - 1)
-    minibuf_error ("End of file during parsing");
+  if (n >= LONG_MAX - 1)
+    ok = leNIL;
+  else
+    {
+      if (n <= 0)
+        n = 1;
+      n--; /* Re-base to counting from zero */
+      if (cur_bp->pt.n > (size_t) n)
+        ngotoup (cur_bp->pt.n - (size_t) n);
+      else if (cur_bp->pt.n < (size_t) n)
+        ngotodown ((size_t) n - cur_bp->pt.n);
+      cur_bp->pt.o = 0;
+    }
 
-  --to_line; /* Re-base to counting from zero */
-  if (cur_bp->pt.n > to_line)
-    ngotoup (cur_bp->pt.n - to_line);
-  else if (cur_bp->pt.n < to_line)
-    ngotodown (to_line -cur_bp->pt.n);
-  cur_bp->pt.o = 0;
-
-  return leT;
+  INT_FREE (n);
 }
 END_DEFUN
 
@@ -294,7 +301,6 @@ Move point to the beginning of the buffer; leave mark at previous position.
 {
   set_mark_interactive ();
   gotobob ();
-  return leT;
 }
 END_DEFUN
 
@@ -315,7 +321,6 @@ Move point to the end of the buffer; leave mark at previous position.
 {
   set_mark_interactive ();
   gotoeob ();
-  return leT;
 }
 END_DEFUN
 
@@ -365,10 +370,9 @@ Move point left N characters (right if N is negative).
 On attempt to pass beginning or end of buffer, stop and signal error.
 +*/
 {
-  le * ret = execute_with_uniarg (false, uniarg, backward_char, forward_char);
-  if (ret == leNIL)
+  ok = execute_with_uniarg (false, uniarg, backward_char, forward_char);
+  if (ok == leNIL)
     minibuf_error ("Beginning of buffer");
-  return ret;
 }
 END_DEFUN
 
@@ -378,10 +382,9 @@ Move point right N characters (left if N is negative).
 On reaching end of buffer, stop and signal error.
 +*/
 {
-  le * ret = execute_with_uniarg (false, uniarg, forward_char, backward_char);
-  if (ret == leNIL)
+  ok = execute_with_uniarg (false, uniarg, forward_char, backward_char);
+  if (ok == leNIL)
     minibuf_error ("End of buffer");
-  return ret;
 }
 END_DEFUN
 
@@ -438,7 +441,7 @@ DEFUN ("scroll-down", scroll_down)
 Scroll text of current window downward near full screen.
 +*/
 {
-  return execute_with_uniarg (false, uniarg, scroll_down, scroll_up);
+  ok = execute_with_uniarg (false, uniarg, scroll_down, scroll_up);
 }
 END_DEFUN
 
@@ -447,6 +450,6 @@ DEFUN ("scroll-up", scroll_up)
 Scroll text of current window upward near full screen.
 +*/
 {
-  return execute_with_uniarg (false, uniarg, scroll_up, scroll_down);
+  ok = execute_with_uniarg (false, uniarg, scroll_up, scroll_down);
 }
 END_DEFUN
