@@ -1,8 +1,6 @@
 /* Key bindings and extended commands
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Sandro Sigala.
-   Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008 Reuben Thomas.
+   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -32,45 +30,38 @@
 #include "gl_array_list.h"
 #include "gl_linked_list.h"
 
-#include "zile.h"
+#include "main.h"
 #include "extern.h"
 
-static Function _last_command;
-
 /*--------------------------------------------------------------------------
  * Key binding.
  *--------------------------------------------------------------------------*/
 
-typedef struct node *node;
-
-struct node
+struct Binding
 {
-  /* The key and the function (the latter is non-NULL only for a leaf). */
-  size_t key;
-  Function func;
+  size_t key; /* The key code (for every level except the root). */
+  Function func; /* The function for this key (if a leaf node). */
 
   /* Branch vector, number of items, max number of items. */
-  node *vec;
+  Binding *vec;
   size_t vecnum, vecmax;
 };
 
-static node root_bindings;
+Binding root_bindings;
 
-static node
+static Binding
 node_new (int vecmax)
 {
-  node p;
-
-  p = (node) XZALLOC (struct node);
+  Binding p = (Binding) XZALLOC (struct Binding);
 
   p->vecmax = vecmax;
-  p->vec = (node *) XCALLOC (vecmax, struct node);
+  p->vec = (Binding *) XCALLOC (vecmax, struct Binding);
 
   return p;
 }
 
-static node
-search_node (node tree, size_t key)
+static Binding
+search_node (Binding tree, size_t key)
 {
   size_t i;
 
@@ -82,15 +73,21 @@ search_node (node tree, size_t key)
 }
 
 static void
-add_node (node tree, node p)
+add_node (Binding tree, Binding p)
 {
   size_t i;
 
+  /* Erase any previous binding the current key might have had in case
+     it was non-prefix and is now being made prefix, as we don't want
+     to accidentally create a default for the prefix map. */
+  if (tree->vecnum == 0)
+    tree->func = NULL;
+
   /* Reallocate vector if there is not enough space. */
   if (tree->vecnum + 1 >= tree->vecmax)
     {
       tree->vecmax += 5;
-      tree->vec = (node *) xrealloc (tree->vec, sizeof (*p) * tree->vecmax);
+      tree->vec = (Binding *) xrealloc (tree->vec, sizeof (*p) * tree->vecmax);
     }
 
   /* Insert the node at the sorted position. */
@@ -108,9 +105,9 @@ add_node (node tree, node p)
 }
 
 static void
-bind_key_vec (node tree, gl_list_t keys, size_t from, Function func)
+bind_key_vec (Binding tree, gl_list_t keys, size_t from, Function func)
 {
-  node p, s = search_node (tree, (size_t) gl_list_get_at (keys, from));
+  Binding p, s = search_node (tree, (size_t) gl_list_get_at (keys, from));
   size_t n = gl_list_size (keys) - from;
 
   if (s == NULL)
@@ -129,27 +126,10 @@ bind_key_vec (node tree, gl_list_t keys, size_t from, Function func)
     s->func = func;
 }
 
-static void
-bind_key_string (node bindings, char *key, Function func)
+static Binding
+search_key (Binding tree, gl_list_t keys, size_t from)
 {
-  gl_list_t keys = keystrtovec (key);
-
-  if (keys && gl_list_size (keys) > 0)
-    bind_key_vec (bindings, keys, 0, func);
-  else
-    {
-      fprintf (stderr,
-               "%s: Key sequence %s is invalid in default bindings\n\n",
-               prog_name, key);
-      exit (1);
-    }
-  gl_list_free (keys);
-}
-
-static node
-search_key (node tree, gl_list_t keys, size_t from)
-{
-  node p = search_node (tree, (size_t) gl_list_get_at (keys, from));
+  Binding p = search_node (tree, (size_t) gl_list_get_at (keys, from));
 
   if (p != NULL)
     {
@@ -163,7 +143,7 @@ search_key (node tree, gl_list_t keys, size_t from)
 }
 
 size_t
-do_completion (astr as)
+do_binding_completion (astr as)
 {
   size_t key;
   astr bs = astr_new ();
@@ -217,30 +197,26 @@ make_completion (gl_list_t keys)
   return astr_cat_char (as, '-');
 }
 
-static node
-completion_scan (node bindings, size_t key, gl_list_t * keys)
+static Function
+completion_scan (Binding bindings, size_t key, gl_list_t * keys)
 {
-  node p;
   *keys = gl_list_create_empty (GL_ARRAY_LIST,
                                 NULL, NULL, NULL, true);
 
   gl_list_add_last (*keys, (void *) key);
 
-  do
+  for (;;)
     {
-      p = search_key (bindings, *keys, 0);
+      astr as;
+      Binding p = search_key (bindings, *keys, 0);
       if (p == NULL)
-        break;
-      if (p->func == NULL)
-        {
-          astr as = make_completion (*keys);
-          gl_list_add_last (*keys, (void *) do_completion (as));
-          astr_delete (as);
-        }
+        return NULL;
+      if (p->func != NULL)
+        return p->func;
+      as = make_completion (*keys);
+      gl_list_add_last (*keys, (void *) do_binding_completion (as));
+      astr_delete (as);
     }
-  while (p->func == NULL);
-
-  return p;
 }
 
 static int
@@ -248,7 +224,7 @@ self_insert_command (void)
 {
   int ret = true;
   /* Mask out ~KBD_CTRL to allow control sequences to be themselves. */
-  int key = lastkey () & ~KBD_CTRL;
+  int key = (int) (lastkey () & ~KBD_CTRL);
   deactivate_mark ();
   if (key <= 0xff)
     {
@@ -272,15 +248,15 @@ Insert the character you type.
 Whichever character you type to run this command is inserted.
 +*/
 {
-  return execute_with_uniarg (true, uniarg, self_insert_command, NULL);
+  ok = execute_with_uniarg (true, uniarg, self_insert_command, NULL);
 }
 END_DEFUN
 
-void
-process_key (size_t key)
-{
-  node p;
+static Function _last_command;
 
+void
+process_key (Binding bindings, size_t key)
+{
   if (key == KBD_NOKEY)
     return;
 
@@ -290,11 +266,11 @@ process_key (size_t key)
   else
     {
       gl_list_t keys;
-      p = completion_scan (root_bindings, key, &keys);
-      if (p != NULL)
+      Function f = completion_scan (bindings, key, &keys);
+      if (f != NULL)
         {
-          p->func (last_uniarg, NULL);
-          _last_command = p->func;
+          f (last_uniarg, NULL);
+          _last_command = f;
         }
       else
         {
@@ -317,14 +293,21 @@ last_command (void)
   return _last_command;
 }
 
-void
+Binding
 init_bindings (void)
+{
+  return node_new (10);
+}
+
+void
+init_default_bindings (void)
 {
   size_t i;
   gl_list_t keys = gl_list_create_empty (GL_ARRAY_LIST,
                                          NULL, NULL, NULL, true);
+  astr as;
 
-  root_bindings = node_new (10);
+  root_bindings = init_bindings ();
 
   /* Bind all printing keys to self_insert_command */
   gl_list_add_last (keys, NULL);
@@ -338,90 +321,196 @@ init_bindings (void)
     }
   gl_list_free (keys);
 
-#define X(c_name, key1)                 \
-  bind_key_string (root_bindings, key1, F_ ## c_name);
-#include "tbl_bind.h"
-#undef X
-}
-
-static void
-recursive_free_bindings (node p)
-{
-  size_t i;
-  for (i = 0; i < p->vecnum; ++i)
-    recursive_free_bindings (p->vec[i]);
-  free (p->vec);
-  free (p);
+  as = astr_new_cstr ("\
+(global-set-key \"\\M-m\" 'back-to-indentation)\
+(global-set-key \"\\C-b\" 'backward-char)\
+(global-set-key \"\\LEFT\" 'backward-char)\
+(global-set-key \"\\BACKSPACE\" 'backward-delete-char)\
+(global-set-key \"\\M-\\BACKSPACE\" 'backward-kill-word)\
+(global-set-key \"\\M-{\" 'backward-paragraph)\
+(global-set-key \"\\C-\\M-b\" 'backward-sexp)\
+(global-set-key \"\\M-b\" 'backward-word)\
+(global-set-key \"\\M-<\" 'beginning-of-buffer)\
+(global-set-key \"\\C-a\" 'beginning-of-line)\
+(global-set-key \"\\HOME\" 'beginning-of-line)\
+(global-set-key \"\\C-xe\" 'call-last-kbd-macro)\
+(global-set-key \"\\M-c\" 'capitalize-word)\
+(global-set-key \"\\M-w\" 'copy-region-as-kill)\
+(global-set-key \"\\C-xrx\" 'copy-to-register)\
+(global-set-key \"\\C-xrs\" 'copy-to-register)\
+(global-set-key \"\\C-x\\C-o\" 'delete-blank-lines)\
+(global-set-key \"\\C-d\" 'delete-char)\
+(global-set-key \"\\DELETE\" 'delete-char)\
+(global-set-key \"\\M-\\\\\" 'delete-horizontal-space)\
+(global-set-key \"\\C-x1\" 'delete-other-windows)\
+(global-set-key \"\\C-x0\" 'delete-window)\
+(global-set-key \"\\C-hb\" 'describe-bindings)\
+(global-set-key \"\\F1b\" 'describe-bindings)\
+(global-set-key \"\\C-hf\" 'describe-function)\
+(global-set-key \"\\F1f\" 'describe-function)\
+(global-set-key \"\\C-hk\" 'describe-key)\
+(global-set-key \"\\F1k\" 'describe-key)\
+(global-set-key \"\\C-hv\" 'describe-variable)\
+(global-set-key \"\\F1v\" 'describe-variable)\
+(global-set-key \"\\C-x\\C-l\" 'downcase-region)\
+(global-set-key \"\\M-l\" 'downcase-word)\
+(global-set-key \"\\C-x)\" 'end-kbd-macro)\
+(global-set-key \"\\M->\" 'end-of-buffer)\
+(global-set-key \"\\C-e\" 'end-of-line)\
+(global-set-key \"\\END\" 'end-of-line)\
+(global-set-key \"\\C-x^\" 'enlarge-window)\
+(global-set-key \"\\C-x\\C-x\" 'exchange-point-and-mark)\
+(global-set-key \"\\M-x\" 'execute-extended-command)\
+(global-set-key \"\\M-q\" 'fill-paragraph)\
+(global-set-key \"\\C-x\\C-v\" 'find-alternate-file)\
+(global-set-key \"\\C-x\\C-f\" 'find-file)\
+(global-set-key \"\\C-x\\C-r\" 'find-file-read-only)\
+(global-set-key \"\\RIGHT\" 'forward-char)\
+(global-set-key \"\\C-f\" 'forward-char)\
+(global-set-key \"\\M-}\" 'forward-paragraph)\
+(global-set-key \"\\C-\\M-f\" 'forward-sexp)\
+(global-set-key \"\\M-f\" 'forward-word)\
+(global-set-key \"\\M-gg\" 'goto-line)\
+(global-set-key \"\\M-g\\M-g\" 'goto-line)\
+(global-set-key \"\\TAB\" 'indent-for-tab-command)\
+(global-set-key \"\\C-xi\" 'insert-file)\
+(global-set-key \"\\C-xrg\" 'insert-register)\
+(global-set-key \"\\C-xri\" 'insert-register)\
+(global-set-key \"\\C-r\" 'isearch-backward)\
+(global-set-key \"\\C-\\M-r\" 'isearch-backward-regexp)\
+(global-set-key \"\\C-s\" 'isearch-forward)\
+(global-set-key \"\\C-\\M-s\" 'isearch-forward-regexp)\
+(global-set-key \"\\M-\\SPC\" 'just-one-space)\
+(global-set-key \"\\C-g\" 'keyboard-quit)\
+(global-set-key \"\\C-xk\" 'kill-buffer)\
+(global-set-key \"\\C-k\" 'kill-line)\
+(global-set-key \"\\C-w\" 'kill-region)\
+(global-set-key \"\\C-\\M-k\" 'kill-sexp)\
+(global-set-key \"\\M-d\" 'kill-word)\
+(global-set-key \"\\C-x\\C-b\" 'list-buffers)\
+(global-set-key \"\\M-h\" 'mark-paragraph)\
+(global-set-key \"\\C-\\M-@\" 'mark-sexp)\
+(global-set-key \"\\C-xh\" 'mark-whole-buffer)\
+(global-set-key \"\\M-@\" 'mark-word)\
+(global-set-key \"\\RET\" 'newline)\
+(global-set-key \"\\C-j\" 'newline-and-indent)\
+(global-set-key \"\\C-n\" 'next-line)\
+(global-set-key \"\\DOWN\" 'next-line)\
+(global-set-key \"\\C-o\" 'open-line)\
+(global-set-key \"\\C-xo\" 'other-window)\
+(global-set-key \"\\INSERT\" 'overwrite-mode)\
+(global-set-key \"\\C-p\" 'previous-line)\
+(global-set-key \"\\UP\" 'previous-line)\
+(global-set-key \"\\M-%\" 'query-replace)\
+(global-set-key \"\\C-q\" 'quoted-insert)\
+(global-set-key \"\\C-l\" 'recenter)\
+(global-set-key \"\\C-x\\C-s\" 'save-buffer)\
+(global-set-key \"\\C-x\\C-c\" 'save-buffers-kill-zile)\
+(global-set-key \"\\C-xs\" 'save-some-buffers)\
+(global-set-key \"\\M-v\" 'scroll-down)\
+(global-set-key \"\\PRIOR\" 'scroll-down)\
+(global-set-key \"\\C-v\" 'scroll-up)\
+(global-set-key \"\\NEXT\" 'scroll-up)\
+(global-set-key \"\\C-xf\" 'set-fill-column)\
+(global-set-key \"\\C-@\" 'set-mark-command)\
+(global-set-key \"\\M-!\" 'shell-command)\
+(global-set-key \"\\M-|\" 'shell-command-on-region)\
+(global-set-key \"\\C-x2\" 'split-window)\
+(global-set-key \"\\C-x(\" 'start-kbd-macro)\
+(global-set-key \"\\C-x\\C-z\" 'suspend-zile)\
+(global-set-key \"\\C-z\" 'suspend-zile)\
+(global-set-key \"\\C-xb\" 'switch-to-buffer)\
+(global-set-key \"\\M-i\" 'tab-to-tab-stop)\
+(global-set-key \"\\C-x\\C-q\" 'toggle-read-only)\
+(global-set-key \"\\C-t\" 'transpose-chars)\
+(global-set-key \"\\C-x\\C-t\" 'transpose-lines)\
+(global-set-key \"\\C-\\M-t\" 'transpose-sexps)\
+(global-set-key \"\\M-t\" 'transpose-words)\
+(global-set-key \"\\C-xu\" 'undo)\
+(global-set-key \"\\C-_\" 'undo)\
+(global-set-key \"\\C-u\" 'universal-argument)\
+(global-set-key \"\\C-x\\C-u\" 'upcase-region)\
+(global-set-key \"\\M-u\" 'upcase-word)\
+(global-set-key \"\\C-h\\C-f\" 'view-zile-FAQ)\
+(global-set-key \"\\F1\\C-f\" 'view-zile-FAQ)\
+(global-set-key \"\\C-hw\" 'where-is)\
+(global-set-key \"\\F1w\" 'where-is)\
+(global-set-key \"\\C-x\\C-w\" 'write-file)\
+(global-set-key \"\\C-y\" 'yank)\
+");
+  lisp_loadstring (as);
+  astr_delete (as);
 }
 
 void
-free_bindings (void)
+free_bindings (Binding binding)
 {
-  recursive_free_bindings (root_bindings);
+  size_t i;
+  for (i = 0; i < binding->vecnum; ++i)
+    free_bindings (binding->vec[i]);
+  free (binding->vec);
+  free (binding);
 }
 
-DEFUN ("global-set-key", global_set_key)
+DEFUN_ARGS ("global-set-key", global_set_key,
+            STR_ARG (keystr)
+            STR_ARG (name))
 /*+
 Bind a command to a key sequence.
 Read key sequence and function name, and bind the function to the key
 sequence.
 +*/
 {
-  int ret = false;
-  size_t key;
   gl_list_t keys;
   Function func;
-  const char *name;
-  size_t argc = countNodes (arglist);
 
-  if (arglist && argc >= 3)
+  STR_INIT (keystr);
+  if (keystr != NULL)
     {
-      keys = keystrtovec (arglist->next->data);
+      keys = keystrtovec (keystr);
       if (keys == NULL)
         {
-          minibuf_error ("Key sequence %s is invalid",
-                         arglist->next->data);
-          return false;
+          minibuf_error ("Key sequence %s is invalid", keystr);
+          return leNIL;
         }
-      name = arglist->next->next->data;
     }
   else
     {
       astr as;
+      size_t key;
 
       minibuf_write ("Set key globally: ");
       key = getkey ();
       completion_scan (root_bindings, key, &keys);
-
       as = keyvectostr (keys);
-      name = minibuf_read_function_name ("Set key %s to command: ",
-                                         astr_cstr (as));
+      keystr = xstrdup (astr_cstr (as));
       astr_delete (as);
     }
 
+  STR_INIT (name)
+  else
+    name = minibuf_read_function_name ("Set key %s to command: ",
+                                       keystr);
   if (name == NULL)
-    return false;
+    return leNIL;
 
   func = get_function (name);
-  if (func)
+  if (func == NULL) /* Possible if called non-interactively */
     {
-      ret = true;
-      bind_key_vec (root_bindings, keys, 0, func);
+      minibuf_error ("No such function `%s'", name);
+      return leNIL;
     }
-  else
-    minibuf_error ("No such function `%s'", name);
+  bind_key_vec (root_bindings, keys, 0, func);
 
-  if (!arglist)
-    free ((char *) name);
   gl_list_free (keys);
-
-  return bool_to_lisp (ret);
+  STR_FREE (keystr);
+  STR_FREE (name);
 }
 END_DEFUN
 
 static void
-walk_bindings_tree (node tree, gl_list_t keys,
-                    void (*process) (astr key, node p, void *st), void *st)
+walk_bindings_tree (Binding tree, gl_list_t keys,
+                    void (*process) (astr key, Binding p, void *st), void *st)
 {
   size_t i, j;
   astr as = chordtostr (tree->key);
@@ -430,7 +519,7 @@ walk_bindings_tree (node tree, gl_list_t keys,
 
   for (i = 0; i < tree->vecnum; ++i)
     {
-      node p = tree->vec[i];
+      Binding p = tree->vec[i];
       if (p->func != NULL)
         {
           astr key = astr_new ();
@@ -453,7 +542,7 @@ walk_bindings_tree (node tree, gl_list_t keys,
 }
 
 static void
-walk_bindings (node tree, void (*process) (astr key, node p, void *st),
+walk_bindings (Binding tree, void (*process) (astr key, Binding p, void *st),
                void *st)
 {
   gl_list_t l = gl_list_create_empty (GL_LINKED_LIST,
@@ -469,7 +558,7 @@ typedef struct
 } gather_bindings_state;
 
 static void
-gather_bindings (astr key, node p, void *st)
+gather_bindings (astr key, Binding p, void *st)
 {
   gather_bindings_state *g = (gather_bindings_state *) st;
 
@@ -489,9 +578,10 @@ message in the buffer.
 +*/
 {
   const char *name = minibuf_read_function_name ("Where is command: ");
-  int ret = false;
   gather_bindings_state g;
 
+  ok = leNIL;
+
   if (name)
     {
       g.f = get_function (name);
@@ -513,38 +603,30 @@ message in the buffer.
               astr_delete (as);
             }
           astr_delete (g.bindings);
-          ret = true;
+          ok = leT;
         }
     }
 
   free ((char *) name);
-  return bool_to_lisp (ret);
 }
 END_DEFUN
 
 const char *
-get_function_by_key_sequence (gl_list_t * keys)
+get_function_by_key (size_t key)
 {
-  node p;
-  size_t c = getkey ();
+  Function f;
+  gl_list_t keys;
 
-  if (c & KBD_META && isdigit ((int) (c & 0xff)))
-    {
-      *keys = gl_list_create_empty (GL_ARRAY_LIST,
-                                    NULL, NULL, NULL, true);
-      gl_list_add_last (*keys, (void *) c);
-      return "universal-argument";
-    }
+  if (key & KBD_META && isdigit ((int) (key & 0xff)))
+    return "universal-argument";
 
-  p = completion_scan (root_bindings, c, keys);
-  if (p == NULL)
-    return NULL;
-  else
-    return get_function_name (p->func);
+  f = completion_scan (root_bindings, key, &keys);
+  gl_list_free (keys);
+  return f ? get_function_name (f) : NULL;
 }
 
 static void
-print_binding (astr key, node p, void *st GCC_UNUSED)
+print_binding (astr key, Binding p, void *st GCC_UNUSED)
 {
   bprintf ("%-15s %s\n", astr_cstr (key), get_function_name (p->func));
 }
@@ -565,6 +647,5 @@ Show a list of all defined keys, and their definitions.
 +*/
 {
   write_temp_buffer ("*Help*", true, write_bindings_list);
-  return leT;
 }
 END_DEFUN
