@@ -1,8 +1,6 @@
 /* Buffer-oriented functions
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Sandro Sigala.
-   Copyright (c) 2003, 2004, 2005, 2006 Reuben Thomas.
+   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -29,7 +27,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "zile.h"
+#include "main.h"
 #include "extern.h"
 
 /*
@@ -151,7 +149,7 @@ set_buffer_filename (Buffer * bp, const char *filename)
 }
 
 /*
- * Search for a buffer named `name'.  If not buffer is found and
+ * Search for a buffer named `name'.  If buffer is not found and
  * the `cflag' variable is set to `true', create a new buffer.
  */
 Buffer *
