@@ -1,7 +1,6 @@
 /* Useful editing functions
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 2004 David A. Capello.
+   Copyright (c) 2004, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -27,7 +26,7 @@
 #include <ctype.h>
 #include "gl_linked_list.h"
 
-#include "zile.h"
+#include "main.h"
 #include "extern.h"
 
 static gl_list_t mark_ring = NULL;	/* Mark ring. */
@@ -132,7 +131,7 @@ int
 eobp (void)
 {
   return (cur_bp->pt.p->next == cur_bp->lines &&
-	  cur_bp->pt.o == astr_len (cur_bp->pt.p->text));
+          cur_bp->pt.o == astr_len (cur_bp->pt.p->text));
 }
 
 /* Return true if point is at the beginning of a line. */
@@ -156,6 +155,6 @@ ding (void)
   if (thisflag & FLAG_DEFINING_MACRO)
     cancel_kbd_macro ();
 
-  if (get_variable_bool ("ring-bell"))
+  if (get_variable_bool ("ring-bell") && cur_wp != NULL)
     term_beep ();
 }
