@@ -1,9 +1,6 @@
 /* Global function prototypes
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Sandro Sigala.
-   Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008 Reuben Thomas.
-   Copyright (c) 2004 David Capello.
+   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -24,6 +21,8 @@
 
 #include "gl_list.h"
 
+#include "xalloc_extra.h"
+
 /* basic.c ---------------------------------------------------------------- */
 size_t get_goalc_bp (Buffer * bp, Point pt);
 size_t get_goalc (void);
@@ -37,13 +36,14 @@ int ngotoup (size_t n);
 int previous_line (void);
 
 /* bind.c ----------------------------------------------------------------- */
-int completion_strcmp (const void *p1, const void *p2);
-size_t do_completion (astr as);
-const char *get_function_by_key_sequence (gl_list_t * keys);
-void process_key (size_t key);
-void init_bindings (void);
+extern Binding root_bindings;
+size_t do_binding_completion (astr as);
+const char *get_function_by_key (size_t key);
+void process_key (Binding bindings, size_t key);
+Binding init_bindings (void);
+void init_default_bindings (void);
 Function last_command (void);
-void free_bindings (void);
+void free_bindings (Binding p);
 
 /* buffer.c --------------------------------------------------------------- */
 int calculate_the_region (Region * rp);
@@ -66,6 +66,7 @@ size_t tab_width (Buffer * bp);
 char *copy_text_block (size_t startn, size_t starto, size_t size);
 
 /* completion.c ----------------------------------------------------------- */
+int completion_strcmp (const void *p1, const void *p2);
 Completion *completion_new (int fileflag);
 void free_completion (Completion * cp);
 void completion_scroll_up (void);
@@ -101,11 +102,15 @@ void init_eval (void);
 void free_eval (void);
 
 /* file.c ----------------------------------------------------------------- */
+extern char coding_eol_lf[3];
+extern char coding_eol_crlf[3];
+extern char coding_eol_cr[3];
+extern char coding_eol_undecided[3];
 int exist_file (const char *filename);
 astr get_home_dir (void);
-astr expand_path (astr path);
+bool expand_path (astr path);
 astr compact_path (astr path);
-int find_file (const char *filename);
+bool find_file (const char *filename);
 void kill_buffer (Buffer * kill_bp);
 void zile_exit (int doabort);
 
@@ -116,6 +121,7 @@ void write_temp_buffer (const char *name, bool show, void (*func) (va_list ap),
 bool delete_region (const Region * r);
 
 /* getkey.c --------------------------------------------------------------- */
+void pushkey (size_t key);
 void ungetkey (size_t key);
 size_t lastkey (void);
 size_t xgetkey (int mode, size_t timeout);
@@ -132,7 +138,8 @@ const char *next_history_element (History * hp);
 
 /* keycode.c -------------------------------------------------------------- */
 astr chordtostr (size_t key);
-gl_list_t keystrtovec (char *key);
+size_t strtochord (const char *buf, size_t * len);
+gl_list_t keystrtovec (const char *key);
 astr keyvectostr (gl_list_t keys);
 
 /* killring.c ------------------------------------------------------------- */
@@ -158,12 +165,14 @@ void free_registers (void);
 /* lisp.c ----------------------------------------------------------------- */
 void init_lisp (void);
 void free_lisp (void);
-bool lisp_load (const char *file);
+void lisp_loadstring (astr as);
+bool lisp_loadfile (const char *file);
 
 /* macro.c ---------------------------------------------------------------- */
 void cancel_kbd_macro (void);
 void add_cmd_to_macro (void);
 void add_key_to_cmd (size_t key);
+void remove_key_from_cmd (void);
 void call_macro (Macro * mp);
 void free_macros (void);
 Macro *get_macro (const char *name);
@@ -189,12 +198,13 @@ void free_minibuf (void);
 void minibuf_error (const char *fmt, ...);
 void minibuf_write (const char *fmt, ...);
 char *minibuf_read (const char *fmt, const char *value, ...);
-size_t minibuf_read_number (const char *fmt, ...);
+unsigned long minibuf_read_number (const char *fmt, ...);
 bool minibuf_test_in_completions (const char *ms, gl_list_t completions);
+int minibuf_read_yn (const char *fmt, ...);
 int minibuf_read_yesno (const char *fmt, ...);
 char *minibuf_read_completion (const char *fmt, char *value, Completion * cp,
                                History * hp, ...);
-char *vminibuf_read_completion (const char *fmt, char *value, Completion * cp,
+char *minibuf_vread_completion (const char *fmt, char *value, Completion * cp,
                                 History * hp, const char *empty_err,
                                 bool (*test) (const char *s, gl_list_t completions),
                                 const char *invalid_err, va_list ap);
@@ -235,9 +245,10 @@ void term_redisplay (void);
 void term_full_redisplay (void);
 void show_splash_screen (const char *splash);
 void term_tidy (void);
+void term_finish (void);
 void term_addnstr (const char *s, size_t len);
 
-/* term_ncurses.c --------------------------------------------------------- */
+/* term_curses.c ---------------------------------------------------------- */
 void term_init (void);
 void term_close (void);
 void term_suspend (void);
@@ -250,6 +261,7 @@ void term_addch (int c);
 void term_attrset (size_t attrs, ...);
 void term_beep (void);
 size_t term_xgetkey (int mode, size_t timeout);
+void term_ungetkey (size_t key);
 
 /* undo.c ----------------------------------------------------------------- */
 extern int undo_nosave;
@@ -262,14 +274,14 @@ void undo_set_unchanged (Undo *up);
 /* variables.c ------------------------------------------------------------ */
 void init_variables (void);
 void free_variables (void);
-bool get_variable_bool (char *var);
-char *minibuf_read_variable_name (char *msg);
+char *minibuf_read_variable_name (char *fmt, ...);
 void set_variable (const char *var, const char *val);
-const char *get_variable_doc (Buffer * bp, char *var, char **defval);
-const char *get_variable_bp (Buffer * bp, char *var);
-const char *get_variable (char *var);
-int get_variable_number_bp (Buffer * bp, char *var);
-int get_variable_number (char *var);
+const char *get_variable_doc (Buffer * bp, const char *var, char **defval);
+const char *get_variable_bp (Buffer * bp, const char *var);
+const char *get_variable (const char *var);
+long get_variable_number_bp (Buffer * bp, const char *var);
+long get_variable_number (const char *var);
+bool get_variable_bool (const char *var);
 
 /* window.c --------------------------------------------------------------- */
 void create_first_window (void);
@@ -280,16 +292,11 @@ void set_current_window (Window * wp);
 void delete_window (Window * del_wp);
 Point window_pt (Window * wp);
 
-/* xalloc.c --------------------------------------------------------------- */
-int xvasprintf (char **ptr, const char *fmt, va_list vargs);
-int xasprintf (char **ptr, const char *fmt, ...);
-void list_free (const void *p);
-
 
 /*
  * Declare external Zile functions.
  */
-#define X(zile_name, c_name, doc)                \
-  extern le *F_ ## c_name (int uniarg, le * l);
+#define X(zile_name, c_name, interactive, doc)   \
+  le *F_ ## c_name (long uniarg, le * l);
 #include "tbl_funcs.h"
 #undef X
