@@ -1,8 +1,6 @@
 /* Kill ring facility functions
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Sandro Sigala.
-   Copyright (c) 2003, 2004 Reuben Thomas.
+   Copyright (c) 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -29,7 +27,7 @@
 #include <string.h>
 #include <ctype.h>
 
-#include "zile.h"
+#include "main.h"
 #include "extern.h"
 
 static astr kill_ring_text;
@@ -140,7 +138,6 @@ With prefix argument, kill that many lines from point.
     ret = execute_with_uniarg (true, uniarg, kill_line_literally, NULL);
 
   deactivate_mark ();
-  return ret;
 }
 END_DEFUN
 
@@ -159,7 +156,7 @@ DEFUN ("kill-region", kill_region)
 /*+
 Kill between point and mark.
 The text is deleted but saved in the kill ring.
-The command C-y (yank) can retrieve it from there.
+The command @kbd{C-y} (yank) can retrieve it from there.
 If the buffer is read-only, Zile will beep and refrain from deleting
 the text, but put the text in the kill ring anyway.  This means that
 you can use the killing commands to copy text from a read-only buffer.
@@ -168,7 +165,7 @@ the text killed this time appends to the text killed last time
 to make one entry in the kill ring.
 +*/
 {
-  return bool_to_lisp (copy_or_kill_the_region (true));
+  ok = bool_to_lisp (copy_or_kill_the_region (true));
 }
 END_DEFUN
 
@@ -177,7 +174,7 @@ DEFUN ("copy-region-as-kill", copy_region_as_kill)
 Save the region as if killed, but don't kill it.
 +*/
 {
-  return bool_to_lisp (copy_or_kill_the_region (false));
+  ok = bool_to_lisp (copy_or_kill_the_region (false));
 }
 END_DEFUN
 
@@ -208,7 +205,7 @@ Kill characters forward until encountering the end of a word.
 With argument, do this that many times.
 +*/
 {
-  return kill (uniarg, F_mark_word);
+  ok = kill (uniarg, F_mark_word);
 }
 END_DEFUN
 
@@ -218,7 +215,7 @@ Kill characters backward until encountering the end of a word.
 With argument, do this that many times.
 +*/
 {
-  return kill (-uniarg, F_mark_word);
+  ok = kill (-uniarg, F_mark_word);
 }
 END_DEFUN
 
@@ -229,7 +226,7 @@ With ARG, kill that many sexps after the cursor.
 Negative arg -N means kill N sexps before the cursor.
 +*/
 {
-  return kill (uniarg, F_mark_sexp);
+  ok = kill (uniarg, F_mark_sexp);
 }
 END_DEFUN
 
@@ -237,7 +234,7 @@ DEFUN ("yank", yank)
 /*+
 Reinsert the last stretch of killed text.
 More precisely, reinsert the stretch of killed text most recently
-killed OR yanked.  Put point at end, and set mark at beginning.
+killed @i{or} yanked.  Put point at end, and set mark at beginning.
 +*/
 {
   if (kill_ring_text == NULL)
@@ -257,7 +254,5 @@ killed OR yanked.  Put point at end, and set mark at beginning.
   undo_nosave = false;
 
   deactivate_mark ();
-
-  return leT;
 }
 END_DEFUN
