@@ -1,8 +1,6 @@
 /* Zile variables handling functions
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Sandro Sigala.
-   Copyright (c) 2003, 2004, 2005, 2006, 2007 Reuben Thomas.
+   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -30,7 +28,7 @@
 #include "hash.h"
 #include "gl_list.h"
 
-#include "zile.h"
+#include "main.h"
 #include "extern.h"
 
 /*
@@ -74,7 +72,7 @@ var_free (void *v)
 static void
 init_builtin_var (const char *var, const char *defval, bool local, const char *doc)
 {
-  var_entry *p = XMALLOC (var_entry);
+  var_entry *p = XZALLOC (var_entry);
   p->var = xstrdup (var);
   p->defval = p->val = xstrdup (defval);
   p->local = local;
@@ -85,8 +83,7 @@ init_builtin_var (const char *var, const char *defval, bool local, const char *d
 static Hash_table *
 new_varlist (void)
 {
-  /* Initial size of 32 is big enough for default variables and some
-     more */
+  /* Initial size is big enough for default variables and some more */
   return hash_initialize (32, NULL, var_hash, var_cmp, var_free);
 }
 
@@ -126,13 +123,13 @@ set_variable_in_list (Hash_table *var_list, const char *var, const char *val)
 void
 set_variable (const char *var, const char *val)
 {
-  struct var_entry *ent, *key = XMALLOC (var_entry);
+  struct var_entry *ent, *key = XZALLOC (var_entry);
   key->var = var;
   ent = hash_lookup (main_vars, key);
   free (key);
-  if (ent->local && cur_bp->vars == NULL)
+  if (ent && ent->local && cur_bp->vars == NULL)
     cur_bp->vars = new_varlist ();
-  set_variable_in_list (ent->local ? cur_bp->vars : main_vars, var, val);
+  set_variable_in_list ((ent && ent->local) ? cur_bp->vars : main_vars, var, val);
 }
 
 void
@@ -142,9 +139,9 @@ free_variables (void)
 }
 
 static var_entry *
-get_variable_entry (Buffer * bp, char *var)
+get_variable_entry (Buffer * bp, const char *var)
 {
-  var_entry *p = NULL, *key = XMALLOC (var_entry);
+  var_entry *p = NULL, *key = XZALLOC (var_entry);
 
   key->var = var;
 
@@ -160,7 +157,7 @@ get_variable_entry (Buffer * bp, char *var)
 }
 
 const char *
-get_variable_doc (Buffer * bp, char *var, char **defval)
+get_variable_doc (Buffer * bp, const char *var, char **defval)
 {
   var_entry *p = get_variable_entry (bp, var);
   if (p != NULL)
@@ -172,38 +169,39 @@ get_variable_doc (Buffer * bp, char *var, char **defval)
 }
 
 const char *
-get_variable_bp (Buffer * bp, char *var)
+get_variable_bp (Buffer * bp, const char *var)
 {
   var_entry *p = get_variable_entry (bp, var);
   return p ? p->val : NULL;
 }
 
 const char *
-get_variable (char *var)
+get_variable (const char *var)
 {
   return get_variable_bp (cur_bp, var);
 }
 
-int
-get_variable_number_bp (Buffer * bp, char *var)
+long
+get_variable_number_bp (Buffer * bp, const char *var)
 {
-  int t = 0;
+  long t = 0;
   const char *s = get_variable_bp (bp, var);
 
   if (s)
-    t = atoi (s);
+    t = strtol (s, NULL, 10);
+  /* FIXME: Check result and signal error. */
 
   return t;
 }
 
-int
-get_variable_number (char *var)
+long
+get_variable_number (const char *var)
 {
   return get_variable_number_bp (cur_bp, var);
 }
 
 bool
-get_variable_bool (char *var)
+get_variable_bool (const char *var)
 {
   const char *p = get_variable (var);
   if (p != NULL)
@@ -213,8 +211,9 @@ get_variable_bool (char *var)
 }
 
 char *
-minibuf_read_variable_name (char *msg)
+minibuf_read_variable_name (char *fmt, ...)
 {
+  va_list ap;
   char *ms;
   Completion *cp = completion_new (false);
   var_entry *p;
@@ -224,45 +223,41 @@ minibuf_read_variable_name (char *msg)
        p = hash_get_next (main_vars, p))
     {
       gl_sortedlist_add (cp->completions, completion_strcmp,
-			 xstrdup (p->var));
+                         xstrdup (p->var));
     }
 
-  ms = vminibuf_read_completion (msg, "", cp, NULL,
-                            "No variable name given",
-                            minibuf_test_in_completions,
-                            "Undefined variable name `%s'", NULL);
+  va_start (ap, fmt);
+  ms = minibuf_vread_completion (fmt, "", cp, NULL,
+                                 "No variable name given",
+                                 minibuf_test_in_completions,
+                                 "Undefined variable name `%s'", ap);
+  va_end (ap);
 
   return ms;
 }
 
-DEFUN ("set-variable", set_variable)
+DEFUN_ARGS ("set-variable", set_variable,
+            STR_ARG (var)
+            STR_ARG (val))
 /*+
 Set a variable value to the user-specified value.
 +*/
 {
-  char *var, *val;
-  size_t argc = countNodes (arglist);
-
-  if (arglist && argc >= 3)
-    set_variable (arglist->next->data, arglist->next->next->data);
+  STR_INIT (var)
   else
-    {
-      var = minibuf_read_variable_name ("Set variable: ");
-      if (var == NULL)
-        return leNIL;
+    var = minibuf_read_variable_name ("Set variable: ");
+  if (var == NULL)
+    return leNIL;
+  STR_INIT (val)
+  else
+    val = minibuf_read ("Set %s to value: ", "", var);
+  if (val == NULL)
+    ok = FUNCALL (keyboard_quit);
 
-      val = minibuf_read ("Set %s to value: ", "", var);
-      if (val == NULL)
-        {
-	  free (var);
-	  return FUNCALL (keyboard_quit);
-        }
+  if (ok == leT)
+    set_variable (var, val);
 
-      set_variable (var, val);
-      free (var);
-      free (val);
-    }
-
-  return leT;
+  STR_FREE (var);
+  STR_FREE (val);
 }
 END_DEFUN
